/*
 * joy.c - Joystick support for MS-DOS.
 *
 * Written by
 *  Ettore Perazzoli <ettore@comm2000.it>
 *
 * This file is part of VICE, the Versatile Commodore Emulator.
 * See README for copyright notice.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 *
 */

#include "vice.h"

#include <stdio.h>
#include <tamtypes.h>

#include "cmdline.h"
#include "keyboard.h"
#include "joy.h"
#include "joystick.h"
#include "resources.h"
#include "types.h"
#include "libpad.h"
#include "pad.h"

int joystick_port_map[2];
int swap_joysticks = 0;
int analog_joysticks = 1;
int use_right_joysticks = 1;

int joystick1_l_h_middle = 127;
int joystick1_l_v_middle = 127;
int joystick1_r_h_middle = 127;
int joystick1_r_v_middle = 127;

int joystick2_l_h_middle = 127;
int joystick2_l_v_middle = 127;
int joystick2_r_h_middle = 127;
int joystick2_r_v_middle = 127;

int joystick_sensitivity = 35;

static int joyport1select(resource_value_t v, void *param)
{
    joystick_port_map[0] = (int)v;
    return 0;
}

static int joyport2select(resource_value_t v, void *param)
{
    joystick_port_map[1] = (int)v;
    return 0;
}

static const resource_int_t resources[] = {
    { "JoyDevice1", JOYDEV_NONE, RES_EVENT_NO, NULL,
      &joystick_port_map[0], joyport1select, NULL },
    { "JoyDevice2", JOYDEV_NONE, RES_EVENT_NO, NULL,
      &joystick_port_map[1], joyport1select, NULL },
    { NULL }
};

int joystick_arch_init_resources(void)
{
    return resources_register_int(resources);
}

int joystick_init_cmdline_options(void)
{
  return 0;
}

static int joystick_init_done = 0;

int joy_arch_init(void)
{
    if (joystick_init_done)
        return 0;

    joystick_init_done = 1;
    return 0;
}

void joy_setabs(void) {
  int value = 0;

  if (analog_joysticks)
  {
    if (use_right_joysticks)
    {
      if (buttons1.rjoy_h<joystick1_r_h_middle-joystick_sensitivity) value |= 4;
      if (buttons1.rjoy_h>joystick1_r_h_middle+joystick_sensitivity) value |= 8;
      if (buttons1.rjoy_v<joystick1_r_v_middle-joystick_sensitivity) value |= 1;
      if (buttons1.rjoy_v>joystick1_r_v_middle+joystick_sensitivity) value |= 2;
    }
    else
    {
      if (buttons1.ljoy_h<joystick1_l_h_middle-joystick_sensitivity) value |= 4;
      if (buttons1.ljoy_h>joystick1_l_h_middle+joystick_sensitivity) value |= 8;
      if (buttons1.ljoy_v<joystick1_l_v_middle-joystick_sensitivity) value |= 1;
      if (buttons1.ljoy_v>joystick1_l_v_middle+joystick_sensitivity) value |= 2;
    }
  }
  else
  {
    if (paddata1 & PAD_LEFT)
      value |= 4;
    if (paddata1 & PAD_RIGHT)
      value |= 8;
    if (paddata1 & PAD_UP)
      value |= 1;
    if (paddata1 & PAD_DOWN)
      value |= 2;
  }

  if (use_right_joysticks)
  {
    if (paddata1 & PAD_DOWN)
      value |= 16;
  }
  else
  {
    if (paddata1 & PAD_CROSS)
      value |= 16;
  }

  joystick_set_value_absolute(swap_joysticks?2:1, value);
}

void joy2_setabs(void) {
  int value = 0;

  if (analog_joysticks)
  {
    if (use_right_joysticks)
    {
      if (buttons2.rjoy_h<joystick2_r_h_middle-joystick_sensitivity) value |= 4;
      if (buttons2.rjoy_h>joystick2_r_h_middle+joystick_sensitivity) value |= 8;
      if (buttons2.rjoy_v<joystick2_r_v_middle-joystick_sensitivity) value |= 1;
      if (buttons2.rjoy_v>joystick2_r_v_middle+joystick_sensitivity) value |= 2;
    }
    else
    {
      if (buttons2.ljoy_h<joystick2_l_h_middle-joystick_sensitivity) value |= 4;
      if (buttons2.ljoy_h>joystick2_l_h_middle+joystick_sensitivity) value |= 8;
      if (buttons2.ljoy_v<joystick2_l_v_middle-joystick_sensitivity) value |= 1;
      if (buttons2.ljoy_v>joystick2_l_v_middle+joystick_sensitivity) value |= 2;
    }
  }
  else
  {
    if (paddata2 & PAD_LEFT)
      value |= 4;
    if (paddata2 & PAD_RIGHT)
      value |= 8;
    if (paddata2 & PAD_UP)
      value |= 1;
    if (paddata2 & PAD_DOWN)
      value |= 2;
  }

  if (use_right_joysticks)
  {
    if (paddata2 & PAD_DOWN)
      value |= 16;
  }
  else
  {
    if (paddata2 & PAD_CROSS)
      value |= 16;
  }

  joystick_set_value_absolute(swap_joysticks?1:2, value);
}

void joystick_update(void)
{
}

void joystick_close(void)
{
   return;
}

void joystick_calibrate(void)
{
  show_ack("Calibrate joysticks now");

  joystick1_l_v_middle = buttons1.ljoy_v;
  joystick1_l_h_middle = buttons1.ljoy_h;
  joystick1_r_v_middle = buttons1.rjoy_v;
  joystick1_r_h_middle = buttons1.rjoy_h;

  joystick2_l_v_middle = buttons2.ljoy_v;
  joystick2_l_h_middle = buttons2.ljoy_h;
  joystick2_r_v_middle = buttons2.rjoy_v;
  joystick2_r_h_middle = buttons2.rjoy_h;
}

int joystick_get_sensitivity()
{
  switch(joystick_sensitivity)
  {
    case 5:
      return 0;
      break;
    case 20:
      return 1;
      break;
    case 35:
      return 2;
      break;
    case 50:
      return 3;
      break;
    case 65:
      return 4;
      break;
    default:
      break;
  }
  return 2;
}

void joystick_set_sensitivity(int sens)
{
  switch(sens)
  {
    case 0:
      show_msg("Setting sensitivity: very high");
      joystick_sensitivity = 5;
      break;
    case 1:
      show_msg("Setting sensitivity: high");
      joystick_sensitivity = 20;
      break;
    case 2:
      show_msg("Setting sensitivity: normal");
      joystick_sensitivity = 35;
      break;
    case 3:
      show_msg("Setting sensitivity: low");
      joystick_sensitivity = 50;
      break;
    case 4:
      show_msg("Setting sensitivity: very low");
      joystick_sensitivity = 65;
      break;
    default:
      show_msg("Setting sensitivity: normal");
      joystick_sensitivity = 35;
      break;
  }
}
