/*
 * ps2main.c - PS2 startup.
 *
 * Written by
 *  Rami Rsnen <raipsu@users.sourceforge.net>
 *
 * This file is part of VICE, the Versatile Commodore Emulator.
 * See README for copyright notice.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 *
 */

#include "vice.h"

#include <tamtypes.h>
#include <eefileio.h>
#include <malloc.h>
#include <kernel.h>
#include <compat.h>
#include <sifcmd.h>
#include <sifrpc.h>
#include <stdio.h>
#include <signal.h>
#include <string.h>

#include "log.h"
#include "machine.h"
#include "main.h"
#include "video.h"
#include "libpad.h"
#include "loadmodule.h"
#include "gs.h"
#include "g2.h"
#include "hw.h"
#include "resources.h"
#include "sjpcm.h"
#include "support.h"
#include "ps2kbd.h"
#include "vkeyboard.h"
#include "cdvd_vsync.h"
#include "pad.h"
#include "joy.h"
#include "menu.h"

char **ps2_argv;
int ps2_argc;

char path_root[2048];
char path_prefix[512];

void graph_init() {
  g2_init(PAL_C64);
/*
    disp_w = 440;
    disp_h = 290;
    curmode = T_PAL;
*/
  g2_set_visible_frame(1);
  g2_set_active_frame(0);

  g2_set_fill_color(0, 0, 0);
  g2_fill_rect(0, 0, g2_get_max_x(), g2_get_max_y());
  g2_flip_buffers();
  g2_fill_rect(0, 0, g2_get_max_x(), g2_get_max_y());
}

void pcm_init() {
  if (_sifLoadModule("rom0:LIBSD", 0, NULL, 0) < 0) {
    printf ("Failed to load rom0:LIBSD.\n");
  }
  loadmodule_robust("sjpcm.irx");
  if (SjPCM_Init(1) < 0) {
    printf ("SjPCM_Init() failed.\n");
  }
}

void kbd_init() {
  if (loadmodule_robust("npm-usbd.irx"))
    loadmodule_robust("usbd.irx");
  loadmodule_robust("ps2kbd.irx");
}

void mass_init() {
  loadmodule_robust("usbd.irx");
  loadmodule_robust("usbhdfsd.irx");
}

extern FILE *tempfile[4];

FILE *fopen_(const char *c, const char *d, int test) {
  FILE *new = malloc(sizeof(FILE));

  bzero((char*)new, sizeof(FILE));

  printf ("opening '%s' '%s'\n", c, d);

  if (!strncmp(c, "cdfs:", 5) || !strncmp(c, "cdrom0:", 7)) {
    cdvd_access();
  }

  new->_ur = -1;   /* temp count */

  if (!strncmp(c, "temp", 4)) {
    new->_ur = atoi(c+4);
  }

  if (new->_ur < -1 || new->_ur > 4) {
    free(new);
    return NULL;
  }

  new->_flags = 0;

  if (index(d, 'w')!=NULL || index(d, '+')!=NULL) {
    if (new->_ur==-1)
      new->_file = fio_open(c, O_RDWR|O_CREAT);
    new->_flags |= 1;
    new->_r = 0;
    new->_offset = 0;
    new->_p = malloc(new->_r + 1);
    if (new->_ur!=-1) {
      if (tempfile[new->_ur]!=NULL) {
        free(tempfile[new->_ur]->_p);
        free(tempfile[new->_ur]);
      }
      tempfile[new->_ur]=new;
    }
    return new;
  } else {
    if (new->_ur!=-1) {
      free(new);
      if (tempfile[new->_ur]==NULL) {
        return NULL;
      }
      tempfile[new->_ur]->_offset = 0;
      tempfile[new->_ur]->_flags = 0;
      return tempfile[new->_ur];
    }
    new->_file = fio_open(c, O_RDONLY);
  }

  if (new->_file<0) {
    free(new);
    return NULL;
  }
  if (test) {
    fio_close(new->_file);
    free(new);
    return (FILE*)1;
  }

  new->_r = fioLseek(new->_file, 0, SEEK_END);
  new->_p = malloc(new->_r + 1);
  fioLseek(new->_file, 0, SEEK_SET);
  new->_offset = 0;

  if (new->_r < 0 || new->_p == NULL) {
    fio_close(new->_file);
    if (new->_p != NULL)
      free(new->_p);
    free(new);
    return NULL;
  }

  if (new->_r) {
    if (fio_read(new->_file, new->_p, new->_r)!=new->_r) {
      fio_close(new->_file);
      free(new->_p);
      free(new);
      return NULL;
    }
  }

  if (!(new->_flags & 1))
    fio_close(new->_file);

  return new;
}

/*
  drive settings:
    true drive emulation (0=off, 1=on) -> 0

  video settings:
    video standard (0=PAL, 1=NTSC) -> 0

  control settings:
    swap joysticks (0=off, 1=on) -> 0
    snapshot with l1/l2 (0=off, 1=on) -> 1
    snapshot confirmation (0=off, 1=on) -> 1
    enable analog joysticks (0=off, 1=on) -> 1
    use right joysticks (0=off, 1=on) -> 1
    joystick sensitivity (0=very high, 1=high, 2=normal, 3=low, 4=very low) -> 2

  sid settings:
    engine (0=fast sid/1=resid) -> 0
    model (0=6581/1=8580) -> 0
    resid sampling (0=fast/1=interpolate) -> 0
    filters (0=off, 1=on) -> 0
*/

void load_configuration()
{
  int true_drive_emulation_conf = -1;
  int video_standard_conf = -1;
  int swap_joysticks_conf = -1;
  int snapshot_with_l1_l2_conf = -1;
  int snapshot_confirmation_conf = -1;
  int enable_analog_joysticks_conf = -1;
  int use_right_joysticks_conf = -1;
  int joystick_sensitivity_conf = -1;
  int sid_engine_conf = -1;
  int sid_model_conf = -1;
  int resid_sampling_conf = -1;
  int sid_filters_conf = -1;

  char filename[4096];
  strcpy(filename, path_root);
  strcat(filename, ".cfg");

  char buffer[512];
  int tmp_int;

  FILE *file_conf = fopen_(filename, "r", 0);
  if ( file_conf == NULL )
  {
    fclose(filename);
    return;
  }

  show_msg("Loading configuration...");

  while(fgets(buffer, sizeof(buffer), file_conf) != NULL)
  {
    if(strncmp(buffer, "TrueDriveEmulation=0", 20) == 0) true_drive_emulation_conf = 0;
    if(strncmp(buffer, "TrueDriveEmulation=1", 20) == 0) true_drive_emulation_conf = 1;

    if(strncmp(buffer, "VideoStandard=0", 15) == 0) video_standard_conf = 0;
    if(strncmp(buffer, "VideoStandard=1", 15) == 0) video_standard_conf = 1;

    if(strncmp(buffer, "SwapJoysticks=0", 15) == 0) swap_joysticks_conf = 0;
    if(strncmp(buffer, "SwapJoysticks=1", 15) == 0) swap_joysticks_conf = 1;

    if(strncmp(buffer, "SnapshotWithL1L2=0", 18) == 0) snapshot_with_l1_l2_conf = 0;
    if(strncmp(buffer, "SnapshotWithL1L2=1", 18) == 0) snapshot_with_l1_l2_conf = 1;

    if(strncmp(buffer, "SnapshotConfirmation=0", 22) == 0) snapshot_confirmation_conf = 0;
    if(strncmp(buffer, "SnapshotConfirmation=1", 22) == 0) snapshot_confirmation_conf = 1;

    if(strncmp(buffer, "EnableAnalogJoysticks=0", 23) == 0) enable_analog_joysticks_conf = 0;
    if(strncmp(buffer, "EnableAnalogJoysticks=1", 23) == 0) enable_analog_joysticks_conf = 1;

    if(strncmp(buffer, "UseRightJoysticks=0", 19) == 0) use_right_joysticks_conf = 0;
    if(strncmp(buffer, "UseRightJoysticks=1", 19) == 0) use_right_joysticks_conf = 1;

    if(strncmp(buffer, "JoystickSensitivity=0", 21) == 0) joystick_sensitivity_conf = 0;
    if(strncmp(buffer, "JoystickSensitivity=1", 21) == 0) joystick_sensitivity_conf = 1;
    if(strncmp(buffer, "JoystickSensitivity=2", 21) == 0) joystick_sensitivity_conf = 2;
    if(strncmp(buffer, "JoystickSensitivity=3", 21) == 0) joystick_sensitivity_conf = 3;
    if(strncmp(buffer, "JoystickSensitivity=4", 21) == 0) joystick_sensitivity_conf = 4;

    if(strncmp(buffer, "SidEngine=0", 11) == 0) sid_engine_conf = 0;
    if(strncmp(buffer, "SidEngine=1", 11) == 0) sid_engine_conf = 1;

    if(strncmp(buffer, "SidModel=0", 10) == 0) sid_model_conf = 0;
    if(strncmp(buffer, "SidModel=1", 10) == 0) sid_model_conf = 1;

    if(strncmp(buffer, "ResidSampling=0", 15) == 0) resid_sampling_conf = 0;
    if(strncmp(buffer, "ResidSampling=1", 15) == 0) resid_sampling_conf = 1;

    if(strncmp(buffer, "SidFilters=0", 12) == 0) sid_filters_conf = 0;
    if(strncmp(buffer, "SidFilters=1", 12) == 0) sid_filters_conf = 1;
  }

  fclose(filename);

  /* true drive emulation */
  if (true_drive_emulation_conf != -1)
  {
    resources_set_value("DriveTrueEmulation", (resource_value_t)true_drive_emulation_conf);
    true_drive_emulation = true_drive_emulation_conf;
  }

  /* video standard */
  if (video_standard_conf != -1)
  {
    resources_set_value("MachineVideoStandard", (resource_value_t)(-video_standard_conf-1));
  }

  /* swap joysticks */
  if (swap_joysticks_conf != -1)
  {
    swap_joysticks = swap_joysticks_conf;
  }

  /* snapshot with l1/l2 */
  if (snapshot_with_l1_l2_conf != -1)
  {
    quick_snapshot = snapshot_with_l1_l2_conf;
  }

  /* snapshot confirmation */
  if (snapshot_confirmation_conf != -1)
  {
    snapshot_confirmation = snapshot_confirmation_conf;
  }

  /* enable analog joysticks */
  if (enable_analog_joysticks_conf != -1)
  {
    analog_joysticks = enable_analog_joysticks_conf;
  }

  /* use right joysticks */
  if (use_right_joysticks_conf != -1)
  {
    use_right_joysticks = use_right_joysticks_conf;
  }

  /* joysticks sensitivity */
  if (joystick_sensitivity_conf != -1)
  {
    joystick_set_sensitivity(joystick_sensitivity_conf);
  }

  /* sid engine */
  if (sid_engine_conf != -1)
  {
    resources_set_value("SidEngine", (resource_value_t)sid_engine_conf);
  }

  /* model */
  if (sid_model_conf != -1)
  {
    resources_set_value("SidModel", (resource_value_t)sid_model_conf);
  }

  /* resid sampling */
  if (resid_sampling_conf != -1)
  {
    resources_set_value("SidResidSampling", (resource_value_t)resid_sampling_conf);
  }

  /* filters */
  if (sid_filters_conf != -1)
  {
    resources_set_value("SidFilters", (resource_value_t)sid_filters_conf);
  }
}

void save_configuration()
{
  int true_drive_emulation_conf;
  int video_standard_conf;
  int swap_joysticks_conf;
  int snapshot_with_l1_l2_conf;
  int snapshot_confirmation_conf;
  int enable_analog_joysticks_conf;
  int use_right_joysticks_conf;
  int joystick_sensitivity_conf;
  int sid_engine_conf;
  int sid_model_conf;
  int resid_sampling_conf;
  int sid_filters_conf;

  /* true drive emulation */
  resources_get_value("DriveTrueEmulation", &true_drive_emulation_conf);

  /* video standard */
  resources_get_value("MachineVideoStandard", &video_standard_conf);
  video_standard_conf = -video_standard_conf-1;

  /* swap joysticks */
  swap_joysticks_conf = swap_joysticks;

  /* snapshot with l1/l2 */
  snapshot_with_l1_l2_conf = quick_snapshot;

  /* snapshot confirmation */
  snapshot_confirmation_conf = snapshot_confirmation;

  /* enable analog joysticks */
  enable_analog_joysticks_conf = analog_joysticks;

  /* use right joysticks */
  use_right_joysticks_conf = use_right_joysticks;

  /* joystick sensitivity */
  joystick_sensitivity_conf = joystick_get_sensitivity();

  /* sid engine */
  resources_get_value("SidEngine", &sid_engine_conf);

  /* model */
  resources_get_value("SidModel", &sid_model_conf);

  /* resid sampling */
  resources_get_value("SidResidSampling", &resid_sampling_conf);

  /* filters */
  resources_get_value("SidFilters", &sid_filters_conf);

  char filename[4096];
  strcpy(filename, path_root);
  strcat(filename, ".cfg");

  char buffer[512];

  FILE *file_conf = fopen_(filename, "w", 0);

  if(file_conf != NULL)
  {
    if(true_drive_emulation_conf==0||true_drive_emulation_conf==1) {
      fputs("# true drive emulation (0=off, 1=on)\n", file_conf);
      sprintf(buffer, "TrueDriveEmulation=%d\n\n", true_drive_emulation_conf);
      fputs(buffer, file_conf);
    }

    if(video_standard_conf==0||video_standard_conf==1) {
      fputs("# video standard (0=PAL, 1=NTSC)\n", file_conf);
      sprintf(buffer, "VideoStandard=%d\n\n", video_standard_conf);
      fputs(buffer, file_conf);
    }

    if(swap_joysticks_conf==0||swap_joysticks_conf==1) {
      fputs("# swap joysticks (0=off, 1=on)\n", file_conf);
      sprintf(buffer, "SwapJoysticks=%d\n\n", swap_joysticks_conf);
      fputs(buffer, file_conf);
    }

    if(snapshot_with_l1_l2_conf==0||snapshot_with_l1_l2_conf==1) {
      fputs("# snapshot with l1/l2 (0=off, 1=on)\n", file_conf);
      sprintf(buffer, "SnapshotWithL1L2=%d\n\n", snapshot_with_l1_l2_conf);
      fputs(buffer, file_conf);
    }

    if(snapshot_confirmation_conf==0||snapshot_confirmation_conf==1) {
      fputs("# snapshot confirmation (0=off, 1=on)\n", file_conf);
      sprintf(buffer, "SnapshotConfirmation=%d\n\n", snapshot_confirmation_conf);
      fputs(buffer, file_conf);
    }

    if(enable_analog_joysticks_conf==0||enable_analog_joysticks_conf==1) {
      fputs("# enable analog joysticks (0=off, 1=on)\n", file_conf);
      sprintf(buffer, "EnableAnalogJoysticks=%d\n\n", enable_analog_joysticks_conf);
      fputs(buffer, file_conf);
    }

    if(use_right_joysticks_conf==0||use_right_joysticks_conf==1) {
      fputs("# use right joysticks (0=off, 1=on)\n", file_conf);
      sprintf(buffer, "UseRightJoysticks=%d\n\n", use_right_joysticks_conf);
      fputs(buffer, file_conf);
    }

    if(joystick_sensitivity_conf==0||joystick_sensitivity_conf==1||
      joystick_sensitivity_conf==2||joystick_sensitivity_conf==3||
      joystick_sensitivity_conf==4) {
      fputs("# joystick sensitivity (0=very high, 1=high, 2=normal, 3=low, 4=very low)\n", file_conf);
      sprintf(buffer, "JoystickSensitivity=%d\n\n", joystick_sensitivity_conf);
      fputs(buffer, file_conf);
    }

    if(sid_engine_conf==0||sid_engine_conf==1) {
      fputs("# sid engine (0=fast sid/1=resid)\n", file_conf);
      sprintf(buffer, "SidEngine=%d\n\n", sid_engine_conf);
      fputs(buffer, file_conf);
    }

    if(sid_model_conf==0||sid_model_conf==1) {
      fputs("# sid model (0=6581/1=8580)\n", file_conf);
      sprintf(buffer, "SidModel=%d\n\n", sid_model_conf);
      fputs(buffer, file_conf);
    }

    if(resid_sampling_conf==0||resid_sampling_conf==1) {
      fputs("# resid sampling (0=fast/1=interpolate)\n", file_conf);
      sprintf(buffer, "ResidSampling=%d\n\n", resid_sampling_conf);
      fputs(buffer, file_conf);
    }

    if(sid_filters_conf==0||sid_filters_conf==1) {
      fputs("# sid filters (0=off, 1=on)\n", file_conf);
      sprintf(buffer, "SidFilters=%d\n", sid_filters_conf);
      fputs(buffer, file_conf);
    }

    char msg[256];
    sprintf(msg, "Wrote configuration to %s", filename);
    show_msg(msg);
  }

  fclose(file_conf);
}

int main(int argc, char **argv)
{

  ps2_argv = argv; ps2_argc = argc;

  if (argc>=1) {
    char *p;
    strcpy(path_root, argv[0]);

    if ((p = rindex(argv[0], '/'))!=NULL) {
      snprintf(path_prefix, sizeof(path_prefix), "%s", argv[0]);
      p = rindex(path_prefix, '/');
      if (p!=NULL)
        p[1]='\0';
    } else if ((p = rindex(argv[0], '\\'))!=NULL) {
      snprintf(path_prefix, sizeof(path_prefix), "%s", argv[0]);
      p = rindex(path_prefix, '\\');
      if (p!=NULL)
        p[1]='\0';
    } else if ((p = index(argv[0], ':'))!=NULL) {
      snprintf(path_prefix, sizeof(path_prefix), "%s", argv[0]);
      p = index(path_prefix, ':');
      if (p!=NULL)
        p[1]='\0';
    }
  }

//  strcpy(path_prefix, "cdrom0:\\");

  SifInitRpc(0);

  graph_init();

  pad_init();

  _sifLoadModule("rom0:SIO2MAN", 0, NULL, 0);
  _sifLoadModule("rom0:CDVDMAN", 0, NULL, 0);
  _sifLoadModule("rom0:MCMAN", 0, NULL, 0);
  _sifLoadModule("rom0:MCSERV", 0, NULL, 0);

  pcm_init();

  kbd_init();

  mass_init();

  vkeyboard_init();
  ps2_keyboard_init();

  if (!strncmp(path_prefix, "cdrom0:", 6)) {
    cdvd_access();
  }

  if (!strncmp(path_prefix, "cdrom0:", 7)) {
    char buf[512];
    snprintf (buf, sizeof(buf), "cdfs:/%s", path_prefix+8);
    strcpy(path_prefix, buf);
  }

  return main_program(argc, argv);
}

void main_exit(void)
{
    log_message(LOG_DEFAULT, "\nExiting...");

    machine_shutdown();

    putchar ('\n');
}

/* Hacks for p00 file support */
static char last_p00_selected_file[2048];
static char last_p00_selected_filename[2048];
static char last_p00_selected_path[2048];

void ps2_store_p00_file(char* name)
{
  strcpy(last_p00_selected_file, name);
}

void ps2_store_p00_filename(char* name)
{
  strcpy(last_p00_selected_filename, name);
}

void ps2_store_p00_path(char* name)
{
  strcpy(last_p00_selected_path, name);
}

char* ps2_get_p00_file()
{
  return last_p00_selected_filename;
}

char* ps2_get_p00_filename()
{
  return last_p00_selected_filename;
}

char* ps2_get_p00_path()
{
  return last_p00_selected_path;
}
